import sys, os, datetime, json
from reportlab.pdfbase.ttfonts import TTFont
from reportlab.pdfbase.pdfmetrics import registerFont
from rlextra.rml2pdf import rml2pdf
import jsondict
from rlextra.radxml.html_cleaner import cleanBlocks
from rlextra.radxml.xhtml2rml import xhtml2rml
from reportlab.lib.utils import getBytesIO
import preppy

def generate_pdf(json_text, options=None):
    data = json.loads(json_text)
    here = os.path.abspath(os.path.dirname(__file__))

    #wrap it up in something friendlier
    data = jsondict.condJSONSafe(data)

    #make a dictionary to pass into preppy as its namespace.
    #you could pass in any Python objects or variables,
    #as long as the template expressions evaluate
    if options:
        fmt = "long" if options.longformat else "short"
        showBoundary = "1" if options.showBoundary else "0"
    else:
        fmt = 'short'
        showBoundary = "0"
    ns = dict(data=data, format=fmt)

    #we usually put some standard things in the preppy namespace
    ns['DATE_GENERATED'] = datetime.date.today()
    ns['showBoundary'] = showBoundary

    #let it know where it is running; trivial in a script, confusing inside
    #a big web framework, may be used to compute other paths.  In Django
    #this might be relative to your project path,
    ns['RML_DIR'] = here

    #directory for images, PDF backgrounds, logos etc relating to the PDF
    ns['RSRC_DIR'] = os.path.join(ns['RML_DIR'], 'resources')

    #We tell our template to use Preppy's standard quoting mechanism.
    #This means any XML characters (&, <, >) will be automatically
    #escaped within the prep file.
    template = preppy.getModule(os.path.join(here,'rml','invoice.prep'))

    #this hack will allow rmltuils functions to 'know' the default quoting mechanism
    #try:
    #   import builtins as __builtin__
    #except:
    #   import __builtin__
    #__builtin__._preppy_stdQuote = preppy.stdQuote
    rmlText = template.getOutput(ns, quoteFunc=preppy.stdQuote)

    buf = getBytesIO()
    rml2pdf.go(rmlText, outputFileName=buf)
    return buf.getvalue()

if __name__=='__main__':
    from optparse import OptionParser
    usage = "usage: runme.py [--long] myfile.json"
    parser = OptionParser(usage=usage)
    parser.add_option("-l", "--long",
                      action="store_true", dest="longformat", default=False,
                      help="Do long profile (rather than short)")
    parser.add_option("-r","--rml",
                      action="store_true", dest="saverml", default=False,
                      help="save a copy of the generated rml")
    parser.add_option("-s","--showb",
                      action="store_true", dest="showBoundary", default=False,
                      help="tuen on global showBoundary flag")
    parser.add_option("-o", "--output",
                      action="store", dest="output", default='output',
                      help="where to store result")

    options, args = parser.parse_args()

    if len(args) != 1:
        print parser.usage
    else:
        filename = args[0]
        with open(filename,'r') as f:
            pdf = generate_pdf(f.read(), options)
        pdfname = os.path.splitext(os.path.basename(filename))[0]+'.pdf'
        with open(pdfname,'wb') as f:
            f.write(pdf)
            print 'saved %s' % pdfname

    #generate_pdf('cv_data.json')
